/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
//-------------------------------------------------------------------
//  File:     HMissile.cpp
//  Created:  September 09, 2000
//  Author:   Jeremie Allard (Hufo / N.A.A.)
//  Comments: Create and manage Scene.
//-------------------------------------------------------------------
#include "Std.h"
#define __HSCENE__
#include "HDef.h"
#include "HScene.h"

SCENE_API void NewMissile(HTank *tank, float speed)
{
  if (tank->state!=HTank::Living)
  {
		tank->Pos.x=bbArena.a.x+(bbArena.b.x-bbArena.a.x)*rnd();
    tank->Pos.y=0.0f;
		tank->Pos.z=bbArena.a.z+(bbArena.b.z-bbArena.a.z)*rnd();
		tank->Angle=(float)RAD2DEG(atan2(-tank->Pos.x+(bbArena.b.x+bbArena.a.x)*0.5f
		                                   ,-tank->Pos.z+(bbArena.b.z+bbArena.a.z)*0.5f));
    tank->GiveLife();
  }
  else
  {
	  HMissile *m=new HMissile(tank,speed);
	  m->SetNext(lMissiles);
	  lMissiles=m;
  }
}

void CalcExplode(HMissile *m)
{
	m->Explode(false);
	HPoint3D Pos=m->Pos;
	int points=0;
	for (int i=0;i<NTANK;i++)
	{
		bool kill=TTank[i].ApplyExplode(Pos);
		if (&(TTank[i])!=m->source && kill)
			++points;
	}
  if (points>0)
	m->source->AddFrag(points);
}

// NEW COLLISION CODE:
void CalcImpact(HMissile *m,int on_tank)
{
	m->Explode(true);
	HPoint3D Pos=m->Pos;
	int points=0;
	for (int i=0;i<NTANK;i++)
	{
		bool kill;
    if (i==on_tank)
      kill=TTank[i].ApplyImpact(Pos,m->Vit);
     else
      kill=TTank[i].ApplyExplode(Pos);
		if (&(TTank[i])!=m->source && kill)
			++points;
	}
  if (points>0)
	  m->source->AddFrag(points);
}

SCENE_API void UpdateMissile(float t,float dt)
{
	HMissile *m;
	HMissile *last;
	m=lMissiles; last=NULL;
	while (m!=NULL)
	{
		bool kill=false;
		m->Update(t,dt);
		if (  m->Pos.x<=HMissile::bbClip.a.x || m->Pos.x>=HMissile::bbClip.b.x
		   || m->Pos.z<=HMissile::bbClip.a.z || m->Pos.z>=HMissile::bbClip.b.z
		   || m->Pos.y<=HMissile::bbClip.a.y) // || m->Pos.y>=HMissile::bbClip.b.y )
		{
			CalcExplode(m);
			kill=true;
		}
    else
// NEW COLLISION CODE:
    { // test collision on each tank
      int i;
      for (i=0;i<NTANK;i++)
      {
        if (TTank[i].HitTest(m))
        {
			    CalcImpact(m,i);
			    kill=true;
          break;
        }
      }
    }
		if (kill)
		{
			if (last==NULL)
				m=lMissiles=lMissiles->DeleteThis();
			else
			{
				last->DeleteNext();
				m=last->Next();
			}
		}
		else
		{
			last=m;
			m=m->Next();
		}
	}
}

SCENE_API void AffMissile()
{
	HMissile *m;
	m=lMissiles;
	while (m!=NULL)
	{
		m->Aff();
		m=m->Next();
	}
}

SCENE_API void FreeMissile()
{
	while (lMissiles!=NULL)
	{
		lMissiles=lMissiles->DeleteThis();
	}
}

// NEW COLLISION CODE:
SCENE_API void ResolveCollisions() // resolve tank collisions
{
  int i,j;
  for (i=0;i<NTANK-1;i++)
    for (j=i+1;j<NTANK;j++)
      TTank[i].ResolveCollision(&(TTank[j]));
}
