/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
//-------------------------------------------------------------------
//  File:     HTank.cpp
//  Created:  September 09, 2000
//  Author:   Jeremie Allard (Hufo / N.A.A.)
//  Comments: Create and manage Tanks.
//-------------------------------------------------------------------
#include "Std.h"
#include "HDef.h"
#include "HTank.h"
#include "HMissile.h"
//#include "IOFile.h"
//#include "imgLoadBMP.h" // Image loader handler for RTexture/RImage.

HObject HTank::oTank;
HObject HTank::oTurret;
HObject HTank::oGun;

HColor4f* HTank::cTank;
HColor4f* HTank::cTurret;
HColor4f* HTank::cGun;

//HPoint3D HTank::posTank(0.0f,1.0f,0.0f); // gun pos relative to turret
HPoint3D HTank::posTank(0.0f,0.0f,0.0f); // gun pos relative to turret
HPoint3D HTank::posTurret(0.0f,1.8f,0.0f); // turret pos relative to tank
//HPoint3D HTank::posGun(0.0f,2.3f,1.2f); // gun pos relative to turret
HPoint3D HTank::posGun(0.0f,-0.5f,1.2f); // gun pos relative to turret

HBBox HTank::bbTank;
HBBox HTank::bbTurret;
HBBox HTank::bbGun;
HBBox HTank::bbCollide;
float       HTank::r2Collide;
HBBox HTank::bbClip;
HPoint3D HTank::endGun;
HPoint3D HTank::pCenter;

HPoint3D HTank::Grav(0,-50,0);

const float HTank::TurretAngleSpeed=180.0f;
const float HTank::GunAngleSpeed=360.0f;

const float HTank::TurretAngleMin=-140.0f;
const float HTank::TurretAngleMax=140.0f;
const float HTank::GunAngleMin=-70.0f;
const float HTank::GunAngleMax=20.0f;

bool HTank::Load()
{
	HPoint3D pivot=posTank;
	Test(oTank  .LoadLWO("data/tank1.lwo"));
	{
		oTank.CalcBBox(&bbTank);
		pivot.y=posTank.y=-bbTank.a.y;
	  oTank  .Translate(-pivot);
	}
	Test(oTurret.LoadLWO("data/tank1_turret.lwo"));
	pivot+=posTurret;
	     oTurret.Translate(-pivot);
	Test(oGun   .LoadLWO("data/tank1_gun.lwo"));
	{
		oGun.CalcBBox(&bbGun);
		posGun.z= bbGun.a.z                -pivot.z;
		posGun.y=(bbGun.a.y+bbGun.b.y)*0.5f-pivot.y;
	pivot+=posGun;
	     oGun   .Translate(-pivot);
	}

  int i;
  if (oTank.nbs>0)
  {
    cTank=new HColor4f[oTank.nbs];
    for (i=0;i<oTank.nbs;i++)
      cTank[i]=oTank.sf[i].ambient;
  }
  else
    cTank=NULL;

  if (oTurret.nbs>0)
  {
    cTurret=new HColor4f[oTurret.nbs];
    for (i=0;i<oTurret.nbs;i++)
      cTurret[i]=oTurret.sf[i].ambient;
  }
  else
    cTurret=NULL;

  if (oGun.nbs>0)
  {
    cGun=new HColor4f[oGun.nbs];
    for (i=0;i<oGun.nbs;i++)
      cGun[i]=oGun.sf[i].ambient;
  }
  else
    cGun=NULL;

	HVector3D light(0.5f,-1.0f,0.2f); light.Normalize();
	HColor4f clight(1.0f,1.0f,1.0f);
	HColor4f ambient(0.2f,0.2f,0.2f);

	oTank.ApplyLight(light,clight,ambient);
	oTurret.ApplyLight(light,clight,ambient);
	oGun.ApplyLight(light,clight,ambient);

	oTank.CalcBBox(&bbTank);
	oTurret.CalcBBox(&bbTurret);
	oGun.CalcBBox(&bbGun);

	endGun.x=(bbGun.a.x+bbGun.b.x)*0.5f;
	endGun.y=(bbGun.a.y+bbGun.b.y)*0.5f;
	endGun.z=bbGun.b.z;

	pCenter=(bbTank.a+bbTank.b)*0.5f;


  // Collision bbox: tank+turret (gun is not considered in)

  bbCollide=bbTurret; // first put turret
  bbCollide.a+=posTurret; // apply translation
  bbCollide.b+=posTurret;
  bbCollide+=bbTank; // then add tank

  HVector3D rc(max(-bbCollide.a.x,bbCollide.b.x),max(-bbCollide.a.y,bbCollide.b.y),max(-bbCollide.a.z,bbCollide.b.z));
  r2Collide=rc.Length2();

	return r_ok;
}

bool HTank::LoadTexture()
{
	HTank::oTank.LoadTexture();
	HTank::oTurret.LoadTexture();
	HTank::oGun.LoadTexture();
	return r_ok;
}

void HTank::SetArena(HBBox *bb)
{
	bbClip=*bb;

	float hsize=(float)sqrt(sqr(max(bbTank.b.x,-bbTank.a.x))+sqr(max(bbTank.b.z,-bbTank.a.z)));

	bbClip.a.x+=hsize; bbClip.b.x-=hsize;
	bbClip.a.z+=hsize; bbClip.b.z-=hsize;
	bbClip.a.y-=bbTank.a.y; bbClip.b.y-=bbTank.b.y;
}

HTank::HTank()
: state(None), Pos(0,0,0), Speed(0), Angle(0), AngleSpeed(0), TurretAngle(0), GunAngle(0)
, TurretAngleGoal(0), GunAngleGoal(0), Life(0.0f),Frag(0)
{
}

void HTank::GiveLife()
{
  state=Living;
  Life=100.0f;
}

void HTank::Aff()
{
  if (state==None) return;

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

    if (state==RIP)
    {
      glDepthMask(0);
      int i;
      HColor4f color(0,0,0,0.5f);
      for (i=0;i<oTank.nbs;i++)
      {
        oTank.sf[i].flag=(oTank.sf[i].flag&~(HSurface::CTxt|HSurface::DynLighting))|HSurface::Blend;
        oTank.sf[i].ambient=color;
      }
      for (i=0;i<oTurret.nbs;i++)
      {
        oTurret.sf[i].flag=(oTurret.sf[i].flag&~(HSurface::CTxt|HSurface::DynLighting))|HSurface::Blend;
        oTurret.sf[i].ambient=color;
      }
      for (i=0;i<oGun.nbs;i++)
      {
        oGun.sf[i].flag=(oGun.sf[i].flag&~(HSurface::CTxt|HSurface::DynLighting))|HSurface::Blend;
        oGun.sf[i].ambient=color;
      }
      // we put the tank on ground -> apply a very low scale on y axis
      glScalef(1.0f,0.1f,1.0f);
    }
    else
    {
      int i;
      for (i=0;i<oTank.nbs;i++)
      {
        oTank.sf[i].flag=(oTank.sf[i].flag|(HSurface::CTxt|HSurface::DynLighting))&~HSurface::Blend;
        oTank.sf[i].ambient=cTank[i];
      }
      for (i=0;i<oTurret.nbs;i++)
      {
        oTurret.sf[i].flag=(oTurret.sf[i].flag|(HSurface::CTxt|HSurface::DynLighting))&~HSurface::Blend;
        oTurret.sf[i].ambient=cTurret[i];
      }
      for (i=0;i<oGun.nbs;i++)
      {
        oGun.sf[i].flag=(oGun.sf[i].flag|(HSurface::CTxt|HSurface::DynLighting))&~HSurface::Blend;
        oGun.sf[i].ambient=cGun[i];
      }
    }

		glTranslatef(Pos.x+posTank.x,Pos.y+posTank.y,Pos.z+posTank.z);
		glRotatef(Angle,0,1,0); // rotate around Y (horizontal)
		oTank.Aff();

		glTranslatef(posTurret.x,posTurret.y,posTurret.z);
		glRotatef(TurretAngle,0,1,0); // rotate around Y (horizontal)
		oTurret.Aff();

		glTranslatef(posGun.x,posGun.y,posGun.z);
		glRotatef(GunAngle,1,0,0); // rotate around X (vertical)
		oGun.Aff();

    if (state==RIP)
      glDepthMask(1);

	glPopMatrix();
}

void HTank::SetCam(HPoint3D pcam_in_turret) // set camera transformation with camera relative to turret
{
	glMatrixMode(GL_MODELVIEW);
	//glLoadIdentity();

		glRotatef(180,0,1,0);
		glTranslatef(-pcam_in_turret.x,-pcam_in_turret.y,-pcam_in_turret.z);
		glRotatef(-TurretAngleGoal,0,1,0); // rotate around Y (horizontal)
		glTranslatef(-posTurret.x,-posTurret.y,-posTurret.z);
		glRotatef(-Angle,0,1,0); // rotate around Y (horizontal)
		glTranslatef(-Pos.x,-Pos.y,-Pos.z);

}

float HTank::GetCamAngle()
{
	return -180+TurretAngleGoal+Angle;
}

void HTank::DeplTank(float speed)
{
	Speed=speed;
}

void HTank::RotateTank(float da)
{
	AngleSpeed=da;
}

void HTank::RotateTurret(float da)
{
	TurretAngleGoal+=da;
	while (TurretAngleGoal>TurretAngleMax && TurretAngleGoal-360.0f>TurretAngleMin)
	{
		TurretAngleGoal-=360.0f;
	}
	while (TurretAngleGoal<TurretAngleMin && TurretAngleGoal+360.0f<TurretAngleMax)
	{
		TurretAngleGoal+=360.0f;
	}
}

void HTank::RotateGun(float da)
{
	GunAngleGoal+=da;
	if (GunAngleGoal>GunAngleMax)
		GunAngleGoal=GunAngleMax;
	else if (GunAngleGoal<GunAngleMin)
		GunAngleGoal=GunAngleMin;
}

void HTank::Update(float t,float dt)
{
  if (state!=Living) return;

	float a=(float)DEG2RAD(Angle);
	HPoint3D dir((float)sin(a),0,(float)cos(a));

	float frot;

	if (Pos.y>bbClip.a.y)
		frot=0.9f;
	else
	{
		float v=Vit.Length();
		if (v>0)
		{
			float d=dir*Vit/v;
			frot=(d>0)?0.2f+0.3f*d:0.2f;
		}
		else frot=0.2f;
	}
	Vit*=(float)pow(frot,dt);

	dir*=Speed*dt*2.0f;

	Vit+=dir;

	Vit+=Grav*dt;

	//Pos+=dir;
	Pos+=Vit*dt;
	Angle+=AngleSpeed*dt;

	if (Pos.x>bbClip.b.x)
	{
		Pos.x=bbClip.b.x; Vit.x*=-0.9f;
	}
	else if (Pos.x<bbClip.a.x)
	{
		Pos.x=bbClip.a.x; Vit.x*=-0.9f;
	}
	if (Pos.y>bbClip.b.y)
	{
		Pos.y=bbClip.b.y; Vit.y*=-1.0f;
	}
	else if (Pos.y<bbClip.a.y)
	{
		Pos.y=bbClip.a.y; Vit.y*=-0.2f;
	}
	if (Pos.z>bbClip.b.z)
	{
		Pos.z=bbClip.b.z; Vit.z*=-0.9f;
	}
	else if (Pos.z<bbClip.a.z)
	{
		Pos.z=bbClip.a.z; Vit.z*=-0.9f;
	}

	if (TurretAngleGoal<TurretAngle)
	{
		TurretAngle-=TurretAngleSpeed*dt;
		if (TurretAngle<TurretAngleGoal)
			TurretAngle=TurretAngleGoal;
	}
	else if (TurretAngleGoal>TurretAngle)
	{
		TurretAngle+=TurretAngleSpeed*dt;
		if (TurretAngle>TurretAngleGoal)
			TurretAngle=TurretAngleGoal;
	}
	if (TurretAngle<TurretAngleMin)
		TurretAngle=TurretAngleMin;
	else if (TurretAngle>TurretAngleMax)
		TurretAngle=TurretAngleMax;

	if (GunAngleGoal<GunAngle)
	{
		GunAngle-=GunAngleSpeed*dt;
		if (GunAngle<GunAngleGoal)
			GunAngle=GunAngleGoal;
	}
	else if (GunAngleGoal>GunAngle)
	{
		GunAngle+=GunAngleSpeed*dt;
		if (GunAngle>GunAngleGoal)
			GunAngle=GunAngleGoal;
	}
	if (GunAngle<GunAngleMin)
		GunAngle=GunAngleMin;
	else if (GunAngle>GunAngleMax)
		GunAngle=GunAngleMax;
}

void HTank::GetGunPos(HPoint3D *pos)
{
	HPoint3D p,p2;
	float a;
	a=DEG2RAD(GunAngle);
	p.x=endGun.x;
	p.y=endGun.y*(float)cos(a)-endGun.z*(float)sin(a);
	p.z=endGun.y*(float)sin(a)+endGun.z*(float)cos(a);
	p+=posGun;

	a=-DEG2RAD(TurretAngle);
	p2.x=p.x*(float)cos(a)-p.z*(float)sin(a);
	p2.y=p.y;
	p2.z=p.x*(float)sin(a)+p.z*(float)cos(a);
	p2+=posTurret;

	a=-DEG2RAD(Angle);
	p.x=p2.x*(float)cos(a)-p2.z*(float)sin(a);
	p.y=p2.y;
	p.z=p2.x*(float)sin(a)+p2.z*(float)cos(a);
	p+=posTank;
	p+=Pos;

	*pos=p;
}

void HTank::GetGunDir(HPoint3D *dir)
{
	HPoint3D p,p2;
	float a;
	a=DEG2RAD(GunAngle);
	p.x=0;
	p.y=-(float)sin(a);
	p.z=(float)cos(a);

	a=-DEG2RAD(TurretAngle);
	p2.x=p.x*(float)cos(a)-p.z*(float)sin(a);
	p2.y=p.y;
	p2.z=p.x*(float)sin(a)+p.z*(float)cos(a);

	a=-DEG2RAD(Angle);
	p.x=p2.x*(float)cos(a)-p2.z*(float)sin(a);
	p.y=p2.y;
	p.z=p2.x*(float)sin(a)+p2.z*(float)cos(a);

	*dir=p;
}

void HTank::AddFrag(int nf)
{
  Frag+=nf;
}

bool HTank::ApplyExplode(const HPoint3D &Impact, float force)
{
  if (state!=Living) return false;
	HPoint3D v=Pos-Impact;
	float l2=v.Length2();
	//v.y*=4.0f;
	Vit+=v*(force/(l2));
	//float pts=force/(float)sqrt(l2);
	//AddScore(-pts);
	//return pts;
  return false; // a char never die by an explosion
}

bool HTank::ApplyImpact(const HPoint3D &Impact, const HPoint3D &MVit, float force) // MVit= missile depl vector
{
  if (state!=Living) return false;
  Vit+=MVit;
	//HPoint3D v=Pos-Impact;
	//float l2=v.Length2();
	//v.y*=4.0f;
	//Vit+=v*(force/(l2));
	//float pts=force/(float)sqrt(l2);
	//AddScore(-pts);
	//return pts;
  Life-=25.0f;
  if (Life<=0.0f)
  {
    Life=0.0f;
    state=RIP;
    return true;
  }
  else
    return false;
}

// NEW COLLISION CODE:
bool HTank::HitTest(HMissile *m) // test if a missile collide on the tank
{
  if (state!=Living) return false;
  HPoint3D mpos=m->Pos-Pos-posTank;
  return bbCollide.in(mpos);
}

// NEW COLLISION CODE:
void HTank::ResolveCollision(HTank *t2) // test if 2 tank collide & resolve it if necessary
{
  HVector3D v=t2->Pos-Pos;
  if (v.Length2()>=r2Collide) return; // too far to collide
  HVector3D dvit=t2->Vit-Vit;
  float vscal=v*dvit;
  if (vscal>=0.0f) return; // collision (if any) is currently resolving
  // collide?
  // for now yes :)

  // calc dvit on axis v

  dvit=v*(vscal/v.Length2());
  Vit+=dvit;
  t2->Vit-=dvit;

/*  
  // scale Vit on axis v by -0.9
  v.Normalize();
  vscal=Vit*v;
  Vit+=(v*(-1.9f*vscal));
  vscal=t2->Vit*v;
  t2->Vit+=(v*(-1.9f*vscal));
*/
}
