/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
#ifndef __HTANK_H__
#define __HTANK_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "HPoint.h"
#include "HObject.h"

class HTankAI;

class HMissile;

class HTank
{
//protected:
public:

	static HObject oTank;
	static HObject oTurret;
	static HObject oGun;
	static HPoint3D posTank; // tank pos offset
	static HPoint3D posTurret; // turret pos relative to tank
	static HPoint3D posGun; // gun pos relative to turret

  static HColor4f *cTank; // ambiant tank colors
  static HColor4f *cTurret; // ambiant tank colors
  static HColor4f *cGun; // ambiant tank colors

	static HBBox bbTank;
	static HBBox bbTurret;
	static HBBox bbGun;

  static HBBox bbCollide; // tank collision bbox;
  static float r2Collide; // bounding collision sphere radius (squared)

	static HBBox bbClip; // arena clip box

	static HPoint3D endGun; // gun out

	static HPoint3D pCenter; // tank out

	static HPoint3D Grav;

	static const float TurretAngleSpeed;
	static const float GunAngleSpeed;
	static const float TurretAngleMin;
	static const float TurretAngleMax;
	static const float GunAngleMin;
	static const float GunAngleMax;

// Tank hierarchy is:
// Tank
//  |
//  -- Turret
//      |
//      -- Gun

public:

	static bool Load();
	static bool LoadTexture();

	static void SetArena(HBBox *bb);

//protected:

  enum
  {
    None,
    Living,
    RIP
  }
  state;

	HPoint3D Pos;// tank pos
	HPoint3D Vit;// tank depl vector

	float Speed; // tank speed
	float Angle; // tank rotation angle
	float AngleSpeed; // tank rotation speed

	float TurretAngle; // turret angle
	float GunAngle; // gun angle
	float TurretAngleGoal; // turret angle goal (the angle the user want to be)
	float GunAngleGoal; // gun angle goal (the angle the user want to be)
	
	//float CoveredDist;

	//float Score;
  float Life; // life points
  int Frag; // nb frags

public:

	HTank();

  void GiveLife();

	void Aff(); // render tank

	void SetCam(HPoint3D pcam_in_turret); // set camera transformation with camera relative to turret
	float GetCamAngle();

	void DeplTank(float speed);
	void RotateTank(float da);
	void RotateTurret(float da);
	void RotateGun(float da);

	void Update(float t,float dt);

	void GetGunPos(HPoint3D *pos);
	void GetGunDir(HPoint3D *dir);

	void AddFrag(int nf=1);

	float GetLife()
	{
		return Life;
	}

	int GetFrag()
	{
		return Frag;
	}

	bool ApplyExplode(const HPoint3D &Impact, float force=100.0f); // calc explode & return true if dead
	bool ApplyImpact(const HPoint3D &Impact, const HPoint3D &Vit, float force=100.0f); // calc impact & return true if dead

  bool HitTest(HMissile *m); // test if a missile collide on the tank

  void ResolveCollision(HTank *t2); // test if 2 tank collide & resolve it if necessary

	friend class HTankAI;

};

#endif
