/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
//-------------------------------------------------------------------
//  File:     HTank.cpp
//  Created:  September 09, 2000
//  Author:   Jeremie Allard (Hufo / N.A.A.)
//  Comments: Computer Controled Tanks.
//-------------------------------------------------------------------
#include "Std.h"
#include "HDef.h"
#include "HTankAI.h"
#include "HScene.h"

HTankAI_Random::HTankAI_Random(HTank *tk)
: HTankAI(tk), AngleSpeed(0), dAngleSpeed(0), Speed(0), dSpeed(0)
, Turret(0), dTurret(0), Gun(0), dGun(0), shoot_time(-10.0f+rnd())
{
}


void HTankAI_Random::Update(float t,float dt)
{

	float frot=(float)pow(0.9f,dt);

	dAngleSpeed=dAngleSpeed*frot+(rnd()-0.5f)*dt*500.0f;
	AngleSpeed=AngleSpeed*frot+dAngleSpeed*dt;
	if (AngleSpeed<-90.0f)     { AngleSpeed=-90.0f; if (dAngleSpeed<0) dAngleSpeed=0; }
	else if (AngleSpeed>90.0f) { AngleSpeed= 90.0f; if (dAngleSpeed>0) dAngleSpeed=0; }

	if (AngleSpeed<=-15.0f || AngleSpeed>=15.0f)
		tank->RotateTank(AngleSpeed);
	else
		tank->RotateTank(0);

	dSpeed=dSpeed*frot+(rnd()-0.25f)*dt*10.0f;
	Speed=Speed*frot+dSpeed*dt;
	if (Speed<-5.0f) Speed=-5.0f;
	else if (Speed>10.0f) Speed=10.0f;

	if (Speed<=-1.0f || Speed>=1.0f)
		tank->DeplTank(Speed);
	else
		tank->DeplTank(0);

	dTurret=dTurret*frot+(rnd()-0.5f)*dt*500.0f;
	Turret=Turret*frot+dTurret*dt;
	if (Turret<-90) { Turret=-90; if (dTurret<0) dTurret=0; }
	else if (Turret>90) { Turret=90; if (dTurret>0) dTurret=0; }

	dGun=dGun*frot+(rnd()-0.5f)*dt*500.0f;
	Gun=Gun*frot+dGun*dt;
	if (Gun<-50) { Gun=-50; if (dGun<0) dGun=0; }
	else if (Gun>10) { Gun=10; if (dGun>0) dGun=0; }

	tank->RotateTurret(Turret-tank->TurretAngleGoal);
	tank->RotateGun(Gun-tank->GunAngleGoal);

	shoot_time+=dt;
	if (shoot_time>=0.0f)
	{
		NewMissile(tank);
		shoot_time=-rnd()*7.0f;
	}

}

////////////////////////////////////////////////////////////////////////////////


HTankAI_Shoot::HTankAI_Shoot(HTank *tk)
: HTankAI(tk), AngleSpeed(0), dAngleSpeed(0), Speed(0), dSpeed(0)
, iCible(-1), shoot_time(-10.0f+rnd())
{
  iMe=tk-TTank;
}

void HTankAI_Shoot::Update(float t,float dt)
{

  switch (tank->state)
  {
  case HTank::None:
    break;
  case HTank::RIP:
    shoot_time+=dt;
    if (shoot_time>=3.0f)
      NewMissile(tank);
    break;
  case HTank::Living:

    if (iCible==-1 || iCible==iMe || TTank[iCible].state!=HTank::Living)
    {
      iCible=rand()%NTANK;
    }
    bool cible_ok=!(iCible==-1 || iCible==iMe || TTank[iCible].state!=HTank::Living);

    if (cible_ok)
    {

      HPoint3D pCible=TTank[iCible].Pos-tank->Pos;

      float dist=pCible.Length(); // cible dist

      float angle; // cible angle

      angle=(float)RAD2DEG(atan2(pCible.x,pCible.z))-tank->Angle;

      if (dist>100.0f) // too far
      {
        if (fabs(angle)<50.0f)
          Speed=2.0f;
        else
          Speed=0.0f;
      }
    /*  else if (dist<20.0f) // too near
      {
        Speed=0;
      }
    */
      if (fabs(angle)>45.0f)
        AngleSpeed=angle;

      tank->RotateTurret(angle-tank->TurretAngleGoal+rnd()-0.5f);
  
      float vangle; // angle corresponding to dist
      // missile trajectory math:
      // p0 = initial pos
      // v0 = initial velocity
      // g = applied gravity
      // p1 = cible
      // dv/dt=g
      // -> v=v0+g.t
      // -> dp/dt=v=v0+g.t
      // -> p=p0+v0.t+g.t
      // g = ( 0 , -G)
      // -> p.x=p0.x+v0.x*t
      // -> p1.x=p0.x+v0.x*t1
      // -> t1 = (p1.x-p0.x)/v0.x
      // p.y=p0.y+v0.y*t-G*t
      // -> p1.y==p0.y+v0.y*t1-G*t1
      // -> v0.y=(p1.y-p0.y+G*t1)/t1
      // Y=p1.y-p0.y X=p1.x-p0.x
      // -> vy=(Y+G*(X/vx))/(X/vx)
      // -> vy=(Y+G*(X/vx))*(vx/X)
      // -> vy=vx*Y/X+G*X/vx
      // ...

      float x=(float)sqrt(sqr(pCible.x)+sqr(pCible.z))-(HTank::posGun.z+HTank::bbGun.b.z); float x2=x*x;
      //float y=pCible.y;
      float G=-HMissile::Grav.y;
      float S2=100.0f*100.0f;
      //vangle=(float)RAD2DEG(-acos(sqrt((   x2*(S2-2*G*y+sqrt(S2*S2-4*G*G-4*G*S2*y))   )/(   2*S2*(x2+y*y)   ))));
      vangle=(float)RAD2DEG(-acos(sqrt((   (S2+sqrt(S2*S2-G*G*x2))   )/(   2*S2   ))));

      tank->RotateGun(vangle-tank->GunAngleGoal);

      shoot_time+=dt;
      if (fabs(tank->GunAngle-tank->GunAngleGoal)<=1.0f && fabs(tank->TurretAngle-tank->TurretAngleGoal)<=1.0f && shoot_time>=1.0f)
      {
        NewMissile(tank);
        shoot_time=0;
      }
    }

    // Other moves is random

	  float frot=(float)pow(0.9f,dt);
	  dAngleSpeed=dAngleSpeed*frot+(rnd()-0.5f)*dt*500.0f;
	  AngleSpeed=AngleSpeed*frot+dAngleSpeed*dt;
	  if (AngleSpeed<-90.0f)     { AngleSpeed=-90.0f; if (dAngleSpeed<0) dAngleSpeed=0; }
	  else if (AngleSpeed>90.0f) { AngleSpeed= 90.0f; if (dAngleSpeed>0) dAngleSpeed=0; }

	  if (AngleSpeed<=-15.0f || AngleSpeed>=15.0f)
		  tank->RotateTank(AngleSpeed);
	  else
		  tank->RotateTank(0);

	  dSpeed=dSpeed*frot+(rnd()-0.25f)*dt*10.0f;
	  Speed=Speed*frot+dSpeed*dt;
	  if (Speed<-5.0f) Speed=-5.0f;
	  else if (Speed>10.0f) Speed=10.0f;

	  if (Speed<=-1.0f || Speed>=1.0f)
		  tank->DeplTank(Speed);
	  else
		  tank->DeplTank(0);
    break;
  }
}
