/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
//-------------------------------------------------------------------
//	File:		tankApp.cpp
//-------------------------------------------------------------------
#include "Std.h"      // Include the standard header.
#include "RTexture.h" // We use textures, just for kicks.
#include "imgLoadBMP.h" // Image loader handler for RTexture/RImage.

#define MAIN
#define MAIN_GLEXT

#include "HDef.h"
#include "HScene.h"
#include "glExt.h"
#include "glExt_compiled_vertex_array.h"

#include "tankApp.h"
#include <Math/vjCoord.h>

#include <Config/vjConfigChunk.h>
#include <Config/vjParseUtil.h>

bool bClipOnSol=false;

#define ETest(e) e
//#define ETest(e) if ((e)!=r_ok) throw new Exception("Test(" ## e ## " failed"); else

//----------------------------------------------
//  Constructor
//----------------------------------------------

tankApp::tankApp(vjKernel* kern)
  : vjGlApp(kern)
{
  for (int i=0;i<CONF_NTANK;i++)
    {
      mTankKeybName[i]="";
      mTankUserName[i]="";
      mTankUserId[i]=-1;
    }
  // by default user use keyboard to control first tank
  mTankKeybName[0]="Keyboard";
  nbf=0;
}

//----------------------------------------------
//  Destructor
//----------------------------------------------
tankApp::~tankApp (void)
{
  FreeMissile();
}

//----------------------------------------------
// Execute any initialization needed before the API is started
//----------------------------------------------

//#include <time.h>
void tankApp::init()
{

  nbf=0;
  time.init("Time");
  keyb.init("Keyboard");
  ETest(HTank::Load());
  ETest(HMissile::Load());
  ETest(HParticule::Load());
  PartSyst.Alloc(5000);
  
  ETest(oArena.LoadLWO("data/sol1.lwo"));
  ETest(oSky.LoadLWO("data/sky.lwo"));
  
  oArena.CalcBBox(&bbArena);
  bbArena.b.y+=50.0f;
  // adjust texture mapping
  oArena.sf[0].ctxt.ctr=(bbArena.a+bbArena.b)*0.5f;
  oArena.sf[0].ctxt.size=(bbArena.b-bbArena.a);
  oArena.sf[0].ctxt.size.x*=-1;
  oArena.CalcMapping();
  
  HTank::SetArena(&bbArena);
  HMissile::SetArena(&bbArena);
  HMissile::SetPartSyst(&PartSyst);
  
  for (int i=0;i<NTANK;i++)
  {
    TTank[i].Pos.x=bbArena.a.x+(bbArena.b.x-bbArena.a.x)*rnd();
    TTank[i].Pos.z=bbArena.a.z+(bbArena.b.z-bbArena.a.z)*rnd();
    TTank[i].Angle=(float)RAD2DEG(atan2(-TTank[i].Pos.x+(bbArena.b.x+bbArena.a.x)*0.5f
					,-TTank[i].Pos.z+(bbArena.b.z+bbArena.a.z)*0.5f));
    TTankAI[i]=NULL;
    TTank[i].GiveLife();
  }

  applyConfig();

  printf("init ok\n");
}

//----------------------------------------------
// Execute any initialization needed <b>after</b> API is started
//  but before the drawManager starts the drawing loops.
//----------------------------------------------
void tankApp::apiInit()
{
}

//----------------------------------------------
// Called immediately upon opening a new OpenGL context
//----------------------------------------------
void tankApp::contextInit()
{
  glGetExtension(&glExt_compiled_vertex_array);
  
#if !(NOTEXTURE)
  oArena.LoadTexture();
  HTank::LoadTexture();
  HMissile::LoadTexture();
  HParticule::LoadTexture();
  oSky.LoadTexture();
#endif
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);
}

/** Function to draw the scene
 * PRE: OpenGL state has correct transformation and buffer selected
 * POST: The current scene has been drawn
 */
void tankApp::draw()
{
  int i;
  HTank* cur_tank=&(TTank[0]);

  vjGlUserData* cur_userdata=static_cast<vjGlDrawManager*>(getDrawManager())->currentUserData();
  if (cur_userdata!=NULL)
  {
    vjUser* cur_user=cur_userdata->getUser();
    if (cur_user!=NULL)
    {
      int t=findTankFromUser(cur_user->getId());
      if (t>=0) cur_tank=&(TTank[t]);
    }
  }

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  // Clear the screen
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  
  HPoint3D pCam(0,6,-6);
  cur_tank->SetCam(pCam);
  
  HColor4f ambient(1.0f,1.0f,1.0f);
  HColor4f clight(1.0f,1.0f,1.0f);
  float light0[4]={-1.0f,0.1f,-1.0f,0.0f};
  glLightfv(GL_LIGHT0,GL_AMBIENT,ambient);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,clight);
  glLightfv(GL_LIGHT0,GL_POSITION,light0);
  glEnable(GL_LIGHT0);
  
  float light1[4]={1.0f,-0.1f,0.5f,0.0f};
  glLightfv(GL_LIGHT1,GL_DIFFUSE,clight);
  glLightfv(GL_LIGHT1,GL_POSITION,light1);
  glEnable(GL_LIGHT1);
  
  glDisable(GL_DEPTH_TEST);
  
  oSky.Aff();
  
  if (bClipOnSol)
    glEnable(GL_DEPTH_TEST);
  
  oArena.Aff();
  
  glEnable(GL_DEPTH_TEST);
  
  for(i=0;i<NTANK;i++)
  {
    TTank[i].Aff();
  }
  
  AffMissile();
  
  {
    float cam_angle=DEG2RAD(cur_tank->GetCamAngle());
    
    HPoint3D dx((float)cos(cam_angle),0,-(float)sin(cam_angle));
    HPoint3D dy(0,1,0);
    
    PartSyst.Aff(dx,dy);
  }

  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
}

/// Function called after tracker update but before start of drawing
void tankApp::preFrame()
{
  static float t=0.0f;
  int i;
  float dt=time->getData();

  if (keyb->keyPressed(VJKEY_ESC) && keyb->keyPressed(VJKEY_CTRL)) kill=true;

  for (i=0;i<NTANK;i++)
  {
    if (isConnectedTank(i))
    {

      bool KeyLeft = false;
      bool KeyRight = false;
      bool KeyUp = false;
      bool KeyDown = false;
      
      float da=0;
      
      vjKeyboardInterface & kb=mTankKeyb[i];
      
      HTank * cur_tank = &(TTank[i]);
      
      KeyLeft=kb->keyPressed(VJKEY_LEFT);
      KeyRight=kb->keyPressed(VJKEY_RIGHT);
      KeyUp=kb->keyPressed(VJKEY_UP);
      KeyDown=kb->keyPressed(VJKEY_DOWN);
      
      //printf("%d%d%d%d\n",KeyLeft,KeyRight,KeyUp,KeyDown);
      
      if (KeyLeft) da+=90;
      if (KeyRight) da-=90;
      
      cur_tank->RotateTank(da);
      
      float dp=0;
      
      if (KeyUp) dp+=10.0f;
      if (KeyDown) dp-=5.0f;
      
      cur_tank->DeplTank(dp);
      
      static int lastbt[NTANK]={0};
      int dx,dy,bt;
      
      dx=kb->keyPressed(VJMOUSE_POSX)-keyb->keyPressed(VJMOUSE_NEGX);
      dy=kb->keyPressed(VJMOUSE_POSY)-keyb->keyPressed(VJMOUSE_NEGY);
      bt=kb->keyPressed(VJMBUTTON1);
      

      cur_tank->RotateTurret(-(float)dx*0.5f);
      cur_tank->RotateGun((float)dy*0.5f);
      if (bt && !lastbt[i])
      {
	NewMissile(cur_tank);
      }
      lastbt[i]=bt;
      
      if (kb->keyPressed(VJKEY_ESC) && kb->keyPressed(VJKEY_CTRL)) kill=true;
    }
  }
  for(i=0;i<NTANK;i++)
  {
    if (TTankAI[i]!=NULL)
      TTankAI[i]->Update(t,dt);
    TTank[i].Update(t,dt);
  }
  ResolveCollisions();
  UpdateMissile(t,dt);
  PartSyst.Update(t,dt);
  t+=dt;
}

/// Function called after drawing has been triggered but BEFORE it completes
void tankApp::intraFrame()
{
}

/// Function called before updating trackers but after the frame is drawn
void tankApp::postFrame()
{
  ++nbf;
}

//: Can the handler handle the given chunk?
//! RETURNS: true - Can handle it
//+          false - Can't handle it
bool tankApp::configCanHandle(vjConfigChunk* chunk)
{
  std::string s = chunk->getType();
  printf("tankApp::configCanHandle(%s)\n",s.data());
  if (!vjstrcasecmp (s, "TankApp"))
    return true;
  return false;
}

//: Add the chunk to the configuration
//! PRE: configCanHandle(chunk) == true
//! RETURNS: success
bool tankApp::configAdd(vjConfigChunk* chunk)
{
  printf("tankapp: config chunk received at frame %d\n",nbf);
  for (int i=0;i<CONF_NTANK;i++)
  {
    mTankKeybName[i]=static_cast<std::string>(chunk->getProperty("TankKeyb",i));
    mTankUserName[i]=static_cast<std::string>(chunk->getProperty("TankUsers",i));
    printf("tank[i]: user=\"%s\", keyb=\"%s\"\n",mTankUserName[i].data(),mTankKeybName[i].data());
  }
  applyConfig();
  return true;
}

//: Remove the chunk from the current configuration
//! PRE: configCanHandle(chunk) == true
//!RETURNS: success
bool tankApp::configRemove(vjConfigChunk* chunk)
{
  return true;
}

//----------------------------------------------
// Multi User helper methods
//----------------------------------------------

int tankApp::findTankFromUser(int userId) //: find tank associated with specified user id
{
  for (int i=0;i<CONF_NTANK;i++)
    if (mTankUserId[i]==userId) return i;
  return -1;
}

bool tankApp::isConnectedTank(int i) //: test if a tank is user controlled
{
  if ((unsigned)i>=(unsigned)CONF_NTANK)
    return false;
  return (mTankKeybName[i].length()>0);
}

void tankApp::applyConfig()
{
  int i;

  for (i=0;i<CONF_NTANK;i++)
  {
    if (mTankKeyb[i].getProxyName()!=mTankKeybName[i])
    {
      if (mTankKeybName[i].length()>0)
	mTankKeyb[i].init(mTankKeybName[i]);
    }
    vjUser *u=vjKernel::instance()->getUser(mTankUserName[i]);
    if (u==NULL)
      mTankUserId[i]=-1;
    else
      mTankUserId[i]=u->getId();
  }
  
  for (i=0;i<NTANK;i++)
  {
    if (!isConnectedTank(i))
    {
      //TTankAI[i]=new HTankAI_Random(&(TTank[i]));
      if (TTankAI[i]==NULL)
	TTankAI[i]=new HTankAI_Shoot(&(TTank[i]));
    }
    else
    {
      if (TTankAI[i]!=NULL) delete TTankAI[i];
      TTankAI[i]=NULL;
    }
  }
}

bool tankApp::killRequested()
{
  return kill;
}
