/******* <my-auto-copyright.pl BEGIN do not edit this line> *******
*								  *
*		    NJ Tank for VR/Net Juggler			  *
*		     Version 1.0  10/12/2002			  *
*-----------------------------------------------------------------*
* COPYRIGHT (C) 2002 Jeremie Allard.        ALL RIGHTS RESERVED.  *
*-----------------------------------------------------------------*
*								  *
*  Original Authors:						  *
*   Jeremie Allard						  *
*								  *
*  Contact: Jeremie.Allard@imag.fr				  *
*           Laboratoire Informatique et Distribution		  *
*           ENSIMAG - antenne de Montbonnot			  *
*           ZIRST 51, avenue Jean Kuntzmann			  *
*           38330 MONTBONNOT SAINT MARTIN			  *
*								  *
*    This program  is free software; you can  redistribute it	  *
*    and/or  modify it  under  the terms  of the GNU  General	  *
*    Public  License  as  published   by  the  Free  Software	  *
*    Foundation;  either  version 2  of the  License,  or any	  *
*    later version.						  *
*								  *
*    This program is distributed in  the hope that it will be	  *
*    useful,  but  WITHOUT  ANY  WARRANTY; without  even  the	  *
*    implied  warranty of  MERCHANTABILITY or  FITNESS  FOR A	  *
*    PARTICULAR PURPOSE.   See the GNU General Public License	  *
*    for more details.						  *
*								  *
*    You  should  have received  a  copy of  the GNU  General	  *
*    Public License along with this library; if not, write to	  *
*    the Free  Software  Foundation,  Inc.,  59 Temple Place,	  *
*    Suite 330, Boston, MA 02111-1307 USA			  *
*								  *
******** <my-auto-copyright.pl END do not edit this line> ********/
/*
 * -----------------------------------------------------------------
 * File:          $RCSfile: tankApp.h,v $
 * Date modified: $Date: 2001/04/12 07:12:17 $
 * Version:       $Revision: 1.4 $
 * -----------------------------------------------------------------
 */

#ifndef tankApp_h
#define tankApp_h

// vrjuggler headers

#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <iostream.h>

#include <Kernel/GL/vjGlApp.h>
#include <Kernel/vjDebug.h>

#include <Math/vjMatrix.h>
#include <Math/vjVec3.h>
#include <Math/vjQuat.h>
#include <Kernel/vjDebug.h>

#include <Input/InputManager/vjKeyboardInterface.h>
#include <Input/InputManager/vjAnalogInterface.h>
#include <Input/vjInput/vjKeyboard.h>
#include <Kernel/vjUser.h>

#include <Kernel/vjConfigChunkHandler.h>

//#include <Input/InputManager/vjPosInterface.h>
//#include <Input/InputManager/vjAnalogInterface.h>
//#include <Input/InputManager/vjDigitalInterface.h>

#define NOTEXTURE 0

#include "PTypes.h"
#if !(NOTEXTURE)
#include "RTexture.h"
#endif

//--------------------------------------------------
// Tank game OpenGL application class
//
//---------------------------------------------------
class tankApp : public vjGlApp//, public vjConfigChunkHandler
{
public:
  tankApp(vjKernel* kern);

  virtual ~tankApp (void);

  // Execute any initialization needed before the API is started
  virtual void init();

  // Execute any initialization needed <b>after</b> API is started
  //  but before the drawManager starts the drawing loops.
  virtual void apiInit();

  // Called immediately upon opening a new OpenGL context
  virtual void contextInit();

  /** Function to draw the scene
   * PRE: OpenGL state has correct transformation and buffer selected
   * POST: The current scene has been drawn
   */
  virtual void draw();

  /// Function called after tracker update but before start of drawing
  virtual void preFrame();

  /// Function called after drawing has been triggered but BEFORE it completes
  virtual void intraFrame();

  /// Function called before updating trackers but after the frame is drawn
  virtual void postFrame();

  /// Functions added for dynamic reconfiguration

  //: Can the handler handle the given chunk?
  //! RETURNS: true - Can handle it
  //+          false - Can't handle it
  virtual bool configCanHandle(vjConfigChunk* chunk);

  /// tank specific functions

  virtual bool killRequested();

 protected:
   //: Add the chunk to the configuration
   //! PRE: configCanHandle(chunk) == true
   //! RETURNS: success
   virtual bool configAdd(vjConfigChunk* chunk);

   //: Remove the chunk from the current configuration
   //! PRE: configCanHandle(chunk) == true
   //!RETURNS: success
   virtual bool configRemove(vjConfigChunk* chunk);

   //: Apply config change
   virtual void applyConfig();

protected:

  vjKeyboardInterface keyb;

  vjAnalogInterface time;

  enum { CONF_NTANK=8 };

  std::string mTankKeybName[CONF_NTANK]; //: Keyboard Name of each tank (read for config)

  std::string mTankUserName[CONF_NTANK]; //: User Name of each tank (read for config)

  vjKeyboardInterface mTankKeyb[CONF_NTANK]; //: Keyboard interface of each tank (updated from config)

  int mTankUserId[CONF_NTANK]; //:User Id of each tank (read from config)

  int findTankFromUser(int userId); //: find tank associated with specified user id

  bool isConnectedTank(int i); //: test if a tank is user controlled

  int nbf;

  bool kill;

};

#endif
